Class: Spree::OptionValue
- Inherits:
-
Base
- Object
- Base
- Spree::OptionValue
- Defined in:
- app/models/spree/option_value.rb
Instance Method Summary collapse
-
#presentation_with_option_type ⇒ String
A string representation of all option value and its option type.
-
#touch_all_variants ⇒ Object
Updates the updated_at column on all the variants associated with this option value.
Instance Method Details
#presentation_with_option_type ⇒ String
30 31 32 |
# File 'app/models/spree/option_value.rb', line 30 def presentation_with_option_type "#{option_type.presentation} - #{presentation}" end |
#touch_all_variants ⇒ Object
Updates the updated_at column on all the variants associated with this option value.
24 25 26 |
# File 'app/models/spree/option_value.rb', line 24 def touch_all_variants variants.find_each(&:touch) end |