Class: Spree::PermissionSets::SuperUser

Inherits:
PermissionSets::Base
  • Object
show all
Defined in:
app/models/spree/permission_sets/super_user.rb

Overview

Full permissions for store administration.

This permission set is always added to users with the :admin role.

It grants permission to perform any read or write action on any resource.

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.categoryObject



16
17
18
# File 'app/models/spree/permission_sets/super_user.rb', line 16

def category
  :super_user
end

.privilegeObject



12
13
14
# File 'app/models/spree/permission_sets/super_user.rb', line 12

def privilege
  :other
end

Instance Method Details

#activate!Object



21
22
23
# File 'app/models/spree/permission_sets/super_user.rb', line 21

def activate!
  can :manage, :all
end