Class: Spree::ReimbursementType
- Inherits:
-
Base
- Object
- Base
- Spree::ReimbursementType
- Defined in:
- app/models/spree/reimbursement_type.rb
Direct Known Subclasses
Defined Under Namespace
Modules: ReimbursementHelpers Classes: Credit, Exchange, OriginalPayment, StoreCredit
Constant Summary collapse
- ORIGINAL =
"original"
Class Method Summary collapse
-
.reimburse(_reimbursement, _return_items, _simulate, *_optional_args) ⇒ Object
This method will reimburse the return items based on however it child implements it By default it takes a reimbursement, the return items it needs to reimburse, and if it is a simulation or a real reimbursement.
Class Method Details
.reimburse(_reimbursement, _return_items, _simulate, *_optional_args) ⇒ Object
This method will reimburse the return items based on however it child implements it By default it takes a reimbursement, the return items it needs to reimburse, and if it is a simulation or a real reimbursement. This should return an array
19 20 21 |
# File 'app/models/spree/reimbursement_type.rb', line 19 def self.reimburse(_reimbursement, _return_items, _simulate, *_optional_args) raise "Implement me" end |