Class: Spree::ShippingRateTax
- Extended by:
- DisplayMoney
- Defined in:
- app/models/spree/shipping_rate_tax.rb
Overview
Used to persist shipping rate tax estimations.
Instance Attribute Summary collapse
-
#shipping_rate ⇒ Spree::ShippingRate
The shipping rate to be taxed.
-
#tax_rate ⇒ Spree::TaxRate
The tax rate used to calculate the tax amount.
Instance Method Summary collapse
Methods included from DisplayMoney
Methods inherited from Base
Methods included from Core::Permalinks
#generate_permalink, #save_permalink
Instance Attribute Details
#shipping_rate ⇒ Spree::ShippingRate
The shipping rate to be taxed
9 10 11 |
# File 'app/models/spree/shipping_rate_tax.rb', line 9 def shipping_rate @shipping_rate end |
#tax_rate ⇒ Spree::TaxRate
The tax rate used to calculate the tax amount
9 10 11 |
# File 'app/models/spree/shipping_rate_tax.rb', line 9 def tax_rate @tax_rate end |
Instance Method Details
#absolute_amount ⇒ Object
26 27 28 |
# File 'app/models/spree/shipping_rate_tax.rb', line 26 def absolute_amount amount.abs end |
#label ⇒ Object
19 20 21 22 23 24 |
# File 'app/models/spree/shipping_rate_tax.rb', line 19 def label I18n.t translation_key, scope: "spree.shipping_rate_tax.label", amount: display_absolute_amount, tax_rate_name: tax_rate.name end |