Method: Spree::UserAddressBook#ship_address=
- Defined in:
- app/models/concerns/spree/user_address_book.rb
#ship_address=(address) ⇒ Object
saves address in address book sets address to the default if automatic_default_address is set to true if address is nil, does nothing and returns nil
66 67 68 69 70 71 |
# File 'app/models/concerns/spree/user_address_book.rb', line 66 def ship_address=(address) if address save_in_address_book(address.attributes, Spree::Config.automatic_default_address) end end |