Method: Spree::Variant::PricingOptions.from_price
- Defined in:
- app/models/spree/variant/pricing_options.rb
.from_price(price) ⇒ Spree::Variant::PricingOptions
This creates the correct pricing options for a price, so that we can easily find other prices with the same pricing-relevant attributes and mark them as non-default.
48 49 50 |
# File 'app/models/spree/variant/pricing_options.rb', line 48 def self.from_price(price) new(currency: price.currency, country_iso: price.country_iso) end |